@@ -77,18 +77,23 @@ class StatusHeader(urwid.WidgetWrap):
         self.error = None
         self.offline = None
         self.title = None
+        self.message = None
         self.sync = None
         self.held = None
         self._error = False
         self._offline = False
         self._title = ''
+        self._message = ''
         self._sync = 0
         self._held = 0
         self.held_key = self.app.config.keymap.formatKeys(keymap.LIST_HELD)
 
-    def update(self, title=None, error=None, offline=None, refresh=True, held=None):
+    def update(self, title=None, message=None, error=None,
+               offline=None, refresh=True, held=None):
         if title is not None:
             self.title = title
+        if message is not None:
+            self.message = message
         if error is not None:
             self.error = error
         if offline is not None:
@@ -100,9 +105,11 @@ class StatusHeader(urwid.WidgetWrap):
             self.refresh()
 
     def refresh(self):
-        if self._title != self.title:
+        if (self._title != self.title or self._message != self.message):
             self._title = self.title
-            self.title_widget.set_text(self._title)
+            self._message = self.message
+            t = self.message or self.title
+            self.title_widget.set_text(t)
         if self._held != self.held:
             self._held = self.held
             if self._held:
@@ -128,7 +135,7 @@ class StatusHeader(urwid.WidgetWrap):
 
 class SearchDialog(mywid.ButtonDialog):
     signals = ['search', 'cancel']
-    def __init__(self, app):
+    def __init__(self, app, default):
         self.app = app
         search_button = mywid.FixedButton('Search')
         cancel_button = mywid.FixedButton('Cancel')
@@ -139,16 +146,20 @@ class SearchDialog(mywid.ButtonDialog):
         super(SearchDialog, self).__init__("Search",
                                            "Enter a change number or search string.",
                                            entry_prompt="Search: ",
+                                           entry_text=default,
                                            buttons=[search_button,
-                                                    cancel_button])
+                                                    cancel_button],
+                                           ring=app.ring)
 
     def keypress(self, size, key):
-        r = super(SearchDialog, self).keypress(size, key)
-        commands = self.app.config.keymap.getCommands(r)
+        if not self.app.input_buffer:
+            key = super(SearchDialog, self).keypress(size, key)
+        keys = self.app.input_buffer + [key]
+        commands = self.app.config.keymap.getCommands(keys)
         if keymap.ACTIVATE in commands:
             self._emit('search')
             return None
-        return r
+        return key
 
 # From: cpython/file/2.7/Lib/webbrowser.py with modification to
 # redirect stdin/out/err.
@@ -176,7 +187,6 @@ class BackgroundBrowser(webbrowser.GenericBrowser):
 
 class App(object):
     simple_change_search = re.compile('^(\d+|I[a-fA-F0-9]{40})$')
-    url_change_search = re.compile('^https?://.*$')
 
     def __init__(self, server=None, palette='default', keymap='default',
                  debug=False, verbose=False, disable_sync=False,
@@ -206,12 +216,14 @@ class App(object):
         self.log = logging.getLogger('gertty.App')
         self.log.debug("Starting")
 
+        self.ring = mywid.KillRing()
+        self.input_buffer = []
         webbrowser.register('xdg-open', None, BackgroundBrowser("xdg-open"))
 
         self.fetch_missing_refs = fetch_missing_refs
         self.config.keymap.updateCommandMap()
-        self.search = search.SearchCompiler(self)
-        self.db = db.Database(self)
+        self.search = search.SearchCompiler(self.config.username)
+        self.db = db.Database(self, self.config.dburi, self.search)
         self.sync = sync.Sync(self)
 
         self.screens = []
@@ -265,11 +277,17 @@ class App(object):
 
         self.popup(dialog)
 
+    def clearInputBuffer(self):
+        if self.input_buffer:
+            self.input_buffer = []
+            self.status.update(message='')
+
     def changeScreen(self, widget, push=True):
         self.log.debug("Changing screen to %s" % (widget,))
         self.status.update(error=False, title=widget.title)
         if push:
             self.screens.append(self.loop.widget)
+        self.clearInputBuffer()
         self.loop.widget = widget
 
     def backScreen(self, target_widget=None):
@@ -282,6 +300,7 @@ class App(object):
         self.log.debug("Popping screen to %s" % (widget,))
         if hasattr(widget, 'title'):
             self.status.update(title=widget.title)
+        self.clearInputBuffer()
         self.loop.widget = widget
         self.refresh(force=True)
 
@@ -295,6 +314,7 @@ class App(object):
         self.log.debug("Clearing screen history")
         while self.screens:
             widget = self.screens.pop()
+            self.clearInputBuffer()
             self.loop.widget = widget
 
     def refresh(self, data=None, force=False):
@@ -326,6 +346,7 @@ class App(object):
     def popup(self, widget,
               relative_width=50, relative_height=25,
               min_width=20, min_height=8):
+        self.clearInputBuffer()
         overlay = urwid.Overlay(widget, self.loop.widget,
                                 'center', ('relative', relative_width),
                                 'middle', ('relative', relative_height),
@@ -435,8 +456,8 @@ class App(object):
         except gertty.view.DisplayError as e:
             return self.error(e.message)
 
-    def searchDialog(self):
-        dialog = SearchDialog(self)
+    def searchDialog(self, default):
+        dialog = SearchDialog(self, default)
         urwid.connect_signal(dialog, 'cancel',
             lambda button: self.backScreen())
         urwid.connect_signal(dialog, 'search',
@@ -456,11 +477,9 @@ class App(object):
 
     trailing_filename_re = re.compile('.*(,[a-z]+)')
     def parseInternalURL(self, url):
-        if not self.url_change_search.match(url):
+        if not url.startswith(self.config.url):
             return None
         result = urlparse.urlparse(url)
-        if result.netloc != self.config.hostname:
-            return None
         change = patchset = filename = None
         path = [x for x in result.path.split('/') if x]
         if path:
@@ -503,7 +522,9 @@ class App(object):
         return None
 
     def unhandledInput(self, key):
-        commands = self.config.keymap.getCommands(key)
+        # get commands from buffer
+        keys = self.input_buffer + [key]
+        commands = self.config.keymap.getCommands(keys)
         if keymap.PREV_SCREEN in commands:
             self.backScreen()
         elif keymap.TOP_SCREEN in commands:
@@ -514,24 +535,23 @@ class App(object):
         elif keymap.QUIT in commands:
             self.quit()
         elif keymap.CHANGE_SEARCH in commands:
-            self.searchDialog()
+            self.searchDialog('')
         elif keymap.LIST_HELD in commands:
-            self.doSearch("status:open is:held")
+            self.doSearch("is:held")
         elif key in self.config.dashboards:
             d = self.config.dashboards[key]
             view = view_change_list.ChangeListView(self, d['query'], d['name'])
             self.changeScreen(view)
-
-    def getRepo(self, project_name):
-        local_path = os.path.join(self.config.git_root, project_name)
-        local_root = os.path.abspath(self.config.git_root)
-        assert os.path.commonprefix((local_root, local_path)) == local_root
-        return gitrepo.Repo(self.config.url+'p/'+project_name,
-                            local_path)
+        elif keymap.FURTHER_INPUT in commands:
+            self.input_buffer.append(key)
+            self.status.update(message=''.join(self.input_buffer))
+            return
+        self.clearInputBuffer()
 
     def openURL(self, url):
         self.log.debug("Open URL %s" % url)
         webbrowser.open_new_tab(url)
+        self.loop.screen.clear()
 
     def time(self, dt):
         utc = dt.replace(tzinfo=dateutil.tz.tzutc())
@@ -578,7 +598,7 @@ class App(object):
         return ret
 
     def localCheckoutCommit(self, project_name, commit_sha):
-        repo = self.getRepo(project_name)
+        repo = gitrepo.get_repo(project_name, self.config)
         try:
             repo.checkout(commit_sha)
             dialog = mywid.MessageDialog('Checkout', 'Change checked out in %s' % repo.path)
@@ -591,7 +611,7 @@ class App(object):
         self.popup(dialog, min_height=min_height)
 
     def localCherryPickCommit(self, project_name, commit_sha):
-        repo = self.getRepo(project_name)
+        repo = gitrepo.get_repo(project_name, self.config)
         try:
             repo.cherryPick(commit_sha)
             dialog = mywid.MessageDialog('Cherry-Pick', 'Change cherry-picked in %s' % repo.path)
