@@ -30,6 +30,12 @@ GLOBAL_HELP = (
      "Search for changes"),
     (keymap.LIST_HELD,
      "List held changes"),
+    (keymap.KILL,
+     "Kill to end of line (editing)"),
+    (keymap.YANK,
+     "Yank from kill ring (editing)"),
+    (keymap.YANK_POP,
+     "Replace previous yank from kill ring (editing)"),
     )
 
 class TextButton(urwid.Button):
@@ -74,9 +80,58 @@ class Table(urwid.WidgetWrap):
         for i, widget in enumerate(cells):
             self._w.contents[i][0].contents.append((widget, ('pack', None)))
 
+class KillRing(object):
+    def __init__(self):
+        self.ring = []
+
+    def kill(self, text):
+        self.ring.append(text)
+
+    def yank(self, repeat=False):
+        if not self.ring:
+            return None
+        if repeat:
+            t = self.ring.pop()
+            self.ring.insert(0, t)
+        return self.ring[-1]
+
+class MyEdit(urwid.Edit):
+    def __init__(self, *args, **kw):
+        self.ring = kw.pop('ring', None)
+        if not self.ring:
+            self.ring = KillRing()
+        self.last_yank = None
+        super(MyEdit, self).__init__(*args, **kw)
+
+    def keypress(self, size, key):
+        (maxcol,) = size
+        if self._command_map[key] == keymap.YANK:
+            text = self.ring.yank()
+            if text:
+                self.last_yank = (self.edit_pos, self.edit_pos+len(text))
+                self.insert_text(text)
+            return
+        if self._command_map[key] == keymap.YANK_POP:
+            if not self.last_yank:
+                return
+            text = self.ring.yank(True)
+            if text:
+                self.edit_text = (self.edit_text[:self.last_yank[0]] +
+                                  self.edit_text[self.last_yank[1]:])
+                self.last_yank = (self.edit_pos, self.edit_pos+len(text))
+                self.insert_text(text)
+            return
+        self.last_yank = None
+        if self._command_map[key] == keymap.KILL:
+            text = self.edit_text[self.edit_pos:]
+            self.edit_text = self.edit_text[:self.edit_pos]
+            self.ring.kill(text)
+        return super(MyEdit, self).keypress(size, key)
+
 @mouse_scroll_decorator.ScrollByWheel
 class ButtonDialog(urwid.WidgetWrap):
-    def __init__(self, title, message, entry_prompt=None, entry_text='', buttons=[]):
+    def __init__(self, title, message, entry_prompt=None,
+                 entry_text='', buttons=[], ring=None):
         button_widgets = []
         for button in buttons:
             button_widgets.append(('pack', button))
@@ -84,7 +139,7 @@ class ButtonDialog(urwid.WidgetWrap):
         rows = []
         rows.append(urwid.Text(message))
         if entry_prompt:
-            self.entry = urwid.Edit(entry_prompt, edit_text=entry_text)
+            self.entry = MyEdit(entry_prompt, edit_text=entry_text, ring=ring)
             rows.append(self.entry)
         else:
             self.entry = None
@@ -95,7 +150,7 @@ class ButtonDialog(urwid.WidgetWrap):
 
 class TextEditDialog(urwid.WidgetWrap):
     signals = ['save', 'cancel']
-    def __init__(self, title, prompt, button, text):
+    def __init__(self, title, prompt, button, text, ring=None):
         save_button = FixedButton(button)
         cancel_button = FixedButton('Cancel')
         urwid.connect_signal(save_button, 'click',
@@ -106,7 +161,7 @@ class TextEditDialog(urwid.WidgetWrap):
                           ('pack', cancel_button)]
         button_columns = urwid.Columns(button_widgets, dividechars=2)
         rows = []
-        self.entry = urwid.Edit(edit_text=text, multiline=True)
+        self.entry = MyEdit(edit_text=text, multiline=True, ring=ring)
         rows.append(urwid.Text(prompt))
         rows.append(self.entry)
         rows.append(urwid.Divider())
@@ -144,6 +199,55 @@ class YesNoDialog(ButtonDialog):
             return None
         return r
 
+class SearchableText(urwid.Text):
+    def set_text(self, markup):
+        self._markup = markup
+        super(SearchableText, self).set_text(markup)
+
+    def search(self, search, attribute):
+        if not search:
+            self.set_text(self._markup)
+            return
+        (text, attrs) = urwid.util.decompose_tagmarkup(self._markup)
+        last = 0
+        found = False
+        while True:
+            start = text.find(search, last)
+            if start < 0:
+                break
+            found = True
+            end = start + len(search)
+            i = 0
+            newattrs = []
+            for attr, al in attrs:
+                if i + al <= start:
+                    i += al
+                    newattrs.append((attr, al))
+                    continue
+                if i >= end:
+                    i += al
+                    newattrs.append((attr, al))
+                    continue
+                before = max(start - i, 0)
+                after = max(i + al - end, 0)
+                if before:
+                    newattrs.append((attr, before))
+                newattrs.append((attribute, len(search)))
+                if after:
+                    newattrs.append((attr, after))
+                i += al
+            if i < start:
+                newattrs.append((None, start-i))
+                i += start-i
+            if i < end:
+                newattrs.append((attribute, len(search)))
+            last = start + 1
+            attrs = newattrs
+        self._text = text
+        self._attrib = attrs
+        self._invalidate()
+        return found
+
 class HyperText(urwid.Text):
     _selectable = True
 
@@ -215,7 +319,11 @@ class HyperText(urwid.Text):
     def getPosAtCoords(self, maxcol, col, row):
         trans = self.get_line_translation(maxcol)
         colpos = 0
-        line = trans[row]
+        line = None
+        try:
+            line = trans[row]
+        except IndexError:
+            return None
         for t in line:
             if len(t) == 2:
                 width, pos = t
