@@ -16,15 +16,15 @@ import datetime
 import re
 
 import ply.yacc as yacc
-from sqlalchemy.sql.expression import and_, or_, not_, exists, select
+from sqlalchemy.sql.expression import and_, or_, not_, select, func
 
 import gertty.db
 import gertty.search
 
-from tokenizer import tokens
+from tokenizer import tokens  # NOQA
 
 def SearchParser():
-    precedence = (
+    precedence = (  # NOQA
         ('left', 'NOT', 'NEG'),
     )
 
@@ -47,12 +47,12 @@ def SearchParser():
     def p_boolean_expr(p):
         '''boolean_expr : expression AND expression
                         | expression OR expression'''
-        if p[2] == 'and':
+        if p[2].lower() == 'and':
             p[0] = and_(p[1], p[3])
-        elif p[2] == 'or':
+        elif p[2].lower() == 'or':
             p[0] = or_(p[1], p[3])
         else:
-            raise SyntaxError()
+            raise gertty.search.SearchSyntaxError("Boolean %s not recognized" % p[2])
 
     def p_negative_expr(p):
         '''negative_expr : NOT expression
@@ -76,6 +76,8 @@ def SearchParser():
                 | has_term
                 | is_term
                 | status_term
+                | file_term
+                | limit_term
                 | op_term'''
         p[0] = p[1]
 
@@ -85,34 +87,26 @@ def SearchParser():
                   | USTRING'''
         p[0] = p[1]
 
-    def p_age_unit(p):
-        '''age_unit : SECONDS
-                    | MINUTES
-                    | HOURS
-                    | DAYS
-                    | WEEKS
-                    | MONTHS
-                    | YEARS'''
-        p[0] = p[1]
-
     def p_age_term(p):
-        '''age_term : OP_AGE NUMBER age_unit'''
+        '''age_term : OP_AGE NUMBER string'''
         now = datetime.datetime.utcnow()
-        delta = p[1]
-        unit = p[2]
-        if unit == 'minutes':
+        delta = p[2]
+        unit = p[3]
+        if unit in ['seconds', 'second', 'sec', 's']:
+            pass
+        elif unit in ['minutes', 'minute', 'min', 'm']:
             delta = delta * 60
-        elif unit == 'hours':
+        elif unit in ['hours', 'hour', 'hr', 'h']:
             delta = delta * 60 * 60
-        elif unit == 'days':
-            delta = delta * 60 * 60 * 60
-        elif unit == 'weeks':
-            delta = delta * 60 * 60 * 60 * 7
-        elif unit == 'months':
-            delta = delta * 60 * 60 * 60 * 30
-        elif unit == 'years':
-            delta = delta * 60 * 60 * 60 * 365
-        p[0] = gertty.db.change_table.c.updated < (now-delta)
+        elif unit in ['days', 'day', 'd']:
+            delta = delta * 60 * 60 * 24
+        elif unit in ['weeks', 'week', 'w']:
+            delta = delta * 60 * 60 * 24 * 7
+        elif unit in ['months', 'month', 'mon']:
+            delta = delta * 60 * 60 * 24 * 30
+        elif unit in ['years', 'year', 'y']:
+            delta = delta * 60 * 60 * 24 * 365
+        p[0] = gertty.db.change_table.c.updated < (now-datetime.timedelta(seconds=delta))
 
     def p_change_term(p):
         '''change_term : OP_CHANGE CHANGE_ID
@@ -133,11 +127,18 @@ def SearchParser():
                        gertty.db.account_table.c.name == p[2])
 
     def p_reviewer_term(p):
-        '''reviewer_term : OP_REVIEWER string'''
+        '''reviewer_term : OP_REVIEWER string
+                         | OP_REVIEWER NUMBER'''
         filters = []
         filters.append(gertty.db.approval_table.c.change_key == gertty.db.change_table.c.key)
         filters.append(gertty.db.approval_table.c.account_key == gertty.db.account_table.c.key)
-        if p[2] == 'self':
+        try:
+            number = int(p[2])
+        except:
+            number = None
+        if number is not None:
+            filters.append(gertty.db.account_table.c.id == number)
+        elif p[2] == 'self':
             username = p.parser.username
             filters.append(gertty.db.account_table.c.username == username)
         else:
@@ -157,8 +158,10 @@ def SearchParser():
 
     def p_project_term(p):
         '''project_term : OP_PROJECT string'''
-        #TODO: support regex
-        p[0] = gertty.db.project_table.c.name == p[2]
+        if p[2].startswith('^'):
+            p[0] = func.matches(p[2], gertty.db.project_table.c.name)
+        else:
+            p[0] = gertty.db.project_table.c.name == p[2]
 
     def p_project_key_term(p):
         '''project_key_term : OP_PROJECT_KEY NUMBER'''
@@ -166,18 +169,24 @@ def SearchParser():
 
     def p_branch_term(p):
         '''branch_term : OP_BRANCH string'''
-        #TODO: support regex
-        p[0] = gertty.db.change_table.c.branch == p[2]
+        if p[2].startswith('^'):
+            p[0] = func.matches(p[2], gertty.db.change_table.c.branch)
+        else:
+            p[0] = gertty.db.change_table.c.branch == p[2]
 
     def p_topic_term(p):
         '''topic_term : OP_TOPIC string'''
-        #TODO: support regex
-        p[0] = gertty.db.change_table.c.topic == p[2]
+        if p[2].startswith('^'):
+            p[0] = func.matches(p[2], gertty.db.change_table.c.topic)
+        else:
+            p[0] = gertty.db.change_table.c.topic == p[2]
 
     def p_ref_term(p):
         '''ref_term : OP_REF string'''
-        #TODO: support regex
-        p[0] = gertty.db.change_table.c.branch == p[2][len('refs/heads/'):]
+        if p[2].startswith('^'):
+            p[0] = func.matches(p[2], 'refs/heads/'+gertty.db.change_table.c.branch)
+        else:
+            p[0] = gertty.db.change_table.c.branch == p[2][len('refs/heads/'):]
 
     label_re = re.compile(r'(?P<label>[a-zA-Z0-9_-]+([a-zA-Z]|((?<![-+])[0-9])))'
                           r'(?P<operator>[<>]?=?)(?P<value>[-+]?[0-9]+)'
@@ -216,7 +225,7 @@ def SearchParser():
         '''message_term : OP_MESSAGE string'''
         filters = []
         filters.append(gertty.db.revision_table.c.change_key == gertty.db.change_table.c.key)
-        filters.append(gertty.db.revision_table.c.message == p[2])
+        filters.append(gertty.db.revision_table.c.message.like('%%%s%%' % p[2]))
         s = select([gertty.db.change_table.c.key], correlate=False).where(and_(*filters))
         p[0] = gertty.db.change_table.c.key.in_(s)
 
@@ -249,7 +258,7 @@ def SearchParser():
 
     def p_is_term(p):
         '''is_term : OP_IS string'''
-        #TODO: implement watched, draft
+        #TODO: implement draft
         username = p.parser.username
         if p[2] == 'reviewed':
             filters = []
@@ -271,6 +280,9 @@ def SearchParser():
             p[0] = gertty.db.account_table.c.username == username
         elif p[2] == 'starred':
             p[0] = gertty.db.change_table.c.starred == True
+        elif p[2] == 'held':
+            # A gertty extension
+            p[0] = gertty.db.change_table.c.held == True
         elif p[2] == 'reviewer':
             filters = []
             filters.append(gertty.db.approval_table.c.change_key == gertty.db.change_table.c.key)
@@ -278,9 +290,22 @@ def SearchParser():
             filters.append(gertty.db.account_table.c.username == username)
             s = select([gertty.db.change_table.c.key], correlate=False).where(and_(*filters))
             p[0] = gertty.db.change_table.c.key.in_(s)
+        elif p[2] == 'watched':
+            p[0] = gertty.db.project_table.c.subscribed == True
         else:
             raise gertty.search.SearchSyntaxError('Syntax error: is:%s is not supported' % p[2])
 
+    def p_file_term(p):
+        '''file_term : OP_FILE string'''
+        if p[2].startswith('^'):
+            p[0] = and_(or_(func.matches(p[2], gertty.db.file_table.c.path),
+                            func.matches(p[2], gertty.db.file_table.c.old_path)),
+                        gertty.db.file_table.c.status is not None)
+        else:
+            p[0] = and_(or_(gertty.db.file_table.c.path == p[2],
+                            gertty.db.file_table.c.old_path == p[2]),
+                        gertty.db.file_table.c.status is not None)
+
     def p_status_term(p):
         '''status_term : OP_STATUS string'''
         if p[2] == 'open':
@@ -290,6 +315,16 @@ def SearchParser():
         else:
             p[0] = gertty.db.change_table.c.status == p[2].upper()
 
+    def p_limit_term(p):
+        '''limit_term : OP_LIMIT NUMBER'''
+        # TODO: Implement this.  The sqlalchemy limit call needs to be
+        # applied to the query operation and so can not be returned as
+        # part of the production here.  The information would need to
+        # be returned out-of-band.  In the mean time, since limits are
+        # not as important in gertty, make this a no-op for now so
+        # that it does not produce a syntax error.
+        p[0] = (True == True)
+
     def p_op_term(p):
         'op_term : OP'
         raise SyntaxError()
