@@ -19,6 +19,7 @@ import urlparse
 
 import urwid
 
+from gertty import gitrepo
 from gertty import keymap
 from gertty import mywid
 from gertty import sync
@@ -42,19 +43,22 @@ class EditTopicDialog(mywid.ButtonDialog):
                                               entry_prompt="Topic: ",
                                               entry_text=topic,
                                               buttons=[save_button,
-                                                       cancel_button])
+                                                       cancel_button],
+                                              ring=app.ring)
 
     def keypress(self, size, key):
-        r = super(EditTopicDialog, self).keypress(size, key)
-        commands = self.app.config.keymap.getCommands(r)
+        if not self.app.input_buffer:
+            key = super(EditTopicDialog, self).keypress(size, key)
+        keys = self.app.input_buffer + [key]
+        commands = self.app.config.keymap.getCommands(keys)
         if keymap.ACTIVATE in commands:
             self._emit('save')
             return None
-        return r
+        return key
 
 class CherryPickDialog(urwid.WidgetWrap):
     signals = ['save', 'cancel']
-    def __init__(self, change):
+    def __init__(self, app, change):
         save_button = mywid.FixedButton('Propose Change')
         cancel_button = mywid.FixedButton('Cancel')
         urwid.connect_signal(save_button, 'click',
@@ -65,7 +69,8 @@ class CherryPickDialog(urwid.WidgetWrap):
                           ('pack', cancel_button)]
         button_columns = urwid.Columns(button_widgets, dividechars=2)
         rows = []
-        self.entry = urwid.Edit(edit_text=change.revisions[-1].message, multiline=True)
+        self.entry = mywid.MyEdit(edit_text=change.revisions[-1].message,
+                                  multiline=True, ring=app.ring)
         self.branch_buttons = []
         rows.append(urwid.Text(u"Branch:"))
         for branch in change.project.branches:
@@ -165,7 +170,8 @@ class ReviewDialog(urwid.WidgetWrap):
                 m = revision.getDraftMessage()
             if m:
                 message = m.message
-        self.message = urwid.Edit("Message: \n", edit_text=message, multiline=True)
+        self.message = mywid.MyEdit("Message: \n", edit_text=message,
+                                    multiline=True, ring=app.ring)
         rows.append(self.message)
         rows.append(urwid.Divider())
         rows.append(buttons)
@@ -183,12 +189,14 @@ class ReviewDialog(urwid.WidgetWrap):
         return (approvals, message)
 
     def keypress(self, size, key):
-        r = super(ReviewDialog, self).keypress(size, key)
-        commands = self.app.config.keymap.getCommands(r)
+        if not self.app.input_buffer:
+            key = super(ReviewDialog, self).keypress(size, key)
+        keys = self.app.input_buffer + [key]
+        commands = self.app.config.keymap.getCommands(keys)
         if keymap.PREV_SCREEN in commands:
             self._emit('cancel')
             return None
-        return r
+        return key
 
 class ReviewButton(mywid.FixedButton):
     def __init__(self, revision_row):
@@ -445,7 +453,7 @@ class ChangeView(urwid.WidgetWrap):
         self.created_label = urwid.Text(u'', wrap='clip')
         self.updated_label = urwid.Text(u'', wrap='clip')
         self.status_label = urwid.Text(u'', wrap='clip')
-        self.permalink_label = urwid.Text(u'', wrap='clip')
+        self.permalink_label = mywid.TextButton(u'', on_press=self.openPermalink)
         change_info = []
         change_info_map={'change-data': 'focused-change-data'}
         for l, v in [("Change-Id", urwid.Padding(urwid.AttrMap(self.change_id_label, None,
@@ -464,7 +472,9 @@ class ChangeView(urwid.WidgetWrap):
                      ("Created", self.created_label),
                      ("Updated", self.updated_label),
                      ("Status", self.status_label),
-                     ("Permalink", self.permalink_label),
+                     ("Permalink", urwid.Padding(urwid.AttrMap(self.permalink_label, None,
+                                                               focus_map=change_info_map),
+                                                 width='pack')),
                      ]:
             row = urwid.Columns([(12, urwid.Text(('change-header', l), wrap='clip')), v])
             change_info.append(row)
@@ -504,7 +514,7 @@ class ChangeView(urwid.WidgetWrap):
             change = session.getChange(self.change_key)
             change_number = change.number
             change_id = change.id
-            repo = self.app.getRepo(change.project.name)
+            repo = gitrepo.get_repo(change.project.name, self.app.config)
             for revision in change.revisions:
                 if not repo.hasCommit(revision.parent):
                     missing_revisions.add(revision.parent)
@@ -569,8 +579,8 @@ class ChangeView(urwid.WidgetWrap):
             self.created_label.set_text(('change-data', str(self.app.time(change.created))))
             self.updated_label.set_text(('change-data', str(self.app.time(change.updated))))
             self.status_label.set_text(('change-data', change.status))
-            url = urlparse.urljoin(self.app.config.url, str(change.number))
-            self.permalink_label.set_text(('change-data', url))
+            self.permalink_url = urlparse.urljoin(self.app.config.url, str(change.number))
+            self.permalink_label.text.set_text(('change-data', self.permalink_url))
             self.commit_message.set_text(change.revisions[-1].message)
 
             categories = []
@@ -626,7 +636,7 @@ class ChangeView(urwid.WidgetWrap):
 
             self.refreshDependencies(session, change)
 
-            repo = self.app.getRepo(change.project.name)
+            repo = gitrepo.get_repo(change.project.name, self.app.config)
             # The listbox has both revisions and messages in it (and
             # may later contain the vote table and change header), so
             # keep track of the index separate from the loop.
@@ -780,8 +790,10 @@ class ChangeView(urwid.WidgetWrap):
         return self.app.toggleHeldChange(self.change_key)
 
     def keypress(self, size, key):
-        r = super(ChangeView, self).keypress(size, key)
-        commands = self.app.config.keymap.getCommands(r)
+        if not self.app.input_buffer:
+            key = super(ChangeView, self).keypress(size, key)
+        keys = self.app.input_buffer + [key]
+        commands = self.app.config.keymap.getCommands(keys)
         if keymap.TOGGLE_REVIEWED in commands:
             self.toggleReviewed()
             self.refresh()
@@ -864,10 +876,10 @@ class ChangeView(urwid.WidgetWrap):
         if keymap.CHERRY_PICK_CHANGE in commands:
             self.cherryPickChange()
             return None
-        if r in self.app.config.reviewkeys:
-            self.reviewKey(self.app.config.reviewkeys[r])
+        if key in self.app.config.reviewkeys:
+            self.reviewKey(self.app.config.reviewkeys[key])
             return None
-        return r
+        return key
 
     def diff(self, revision_key):
         if self.app.config.diff_view == 'unified':
@@ -953,7 +965,7 @@ class ChangeView(urwid.WidgetWrap):
     def cherryPickChange(self):
         with self.app.db.getSession() as session:
             change = session.getChange(self.change_key)
-            dialog = CherryPickDialog(change)
+            dialog = CherryPickDialog(self.app, change)
         urwid.connect_signal(dialog, 'cancel', self.app.backScreen)
         urwid.connect_signal(dialog, 'save', lambda button:
                                  self.doCherryPickChange(dialog))
@@ -1013,6 +1025,9 @@ class ChangeView(urwid.WidgetWrap):
         self.app.backScreen()
         self.refresh()
 
+    def openPermalink(self, widget):
+        self.app.openURL(self.permalink_url)
+
     def searchChangeId(self, widget):
         self.app.doSearch("status:open change:%s" % (self.change_id,))
 
