@@ -178,6 +178,10 @@ class ChangeListView(urwid.WidgetWrap):
              "Toggle the starred flag for the currently selected change"),
             (key(keymap.TOGGLE_MARK),
              "Toggle the process mark for the currently selected change"),
+            (key(keymap.REFINE_CHANGE_SEARCH),
+             "Refine the current search query"),
+            (key(keymap.EDIT_TOPIC),
+             "Set the topic of the marked changes"),
             (key(keymap.REFRESH),
              refresh_help),
             (key(keymap.REVIEW),
@@ -290,6 +294,11 @@ class ChangeListView(urwid.WidgetWrap):
             row = self.change_rows[key]
             del self.change_rows[key]
 
+    def getQueryString(self):
+        if self.project_key is not None:
+            return "project:%s %s" % (self.query_desc, self.app.config.project_change_list_query)
+        return self.query
+
     def _threadChanges(self, changes):
         ret = []
         prefixes = {}
@@ -415,8 +424,10 @@ class ChangeListView(urwid.WidgetWrap):
             self.listbox.focus_position = pos
 
     def keypress(self, size, key):
-        r = super(ChangeListView, self).keypress(size, key)
-        commands = self.app.config.keymap.getCommands(r)
+        if not self.app.input_buffer:
+            key = super(ChangeListView, self).keypress(size, key)
+        keys = self.app.input_buffer + [key]
+        commands = self.app.config.keymap.getCommands(keys)
         if keymap.TOGGLE_LIST_REVIEWED in commands:
             self.unreviewed = not self.unreviewed
             self.refresh()
@@ -495,6 +506,9 @@ class ChangeListView(urwid.WidgetWrap):
                 row.update(change, self.categories)
             self.advance()
             return None
+        if keymap.EDIT_TOPIC in commands:
+            self.editTopic()
+            return None
         if keymap.REFRESH in commands:
             if self.project_key:
                 self.app.sync.submitTask(
@@ -549,6 +563,10 @@ class ChangeListView(urwid.WidgetWrap):
             row = self.listbox.body[pos]
             self.app.localCherryPickCommit(row.project_name, row.commit_sha)
             return None
+        if keymap.REFINE_CHANGE_SEARCH in commands:
+            default = self.getQueryString()
+            self.app.searchDialog(default)
+            return None
         return key
 
     def onSelect(self, button, change_key):
@@ -581,3 +599,28 @@ class ChangeListView(urwid.WidgetWrap):
                     sync.UploadReviewTask(message_key, sync.HIGH_PRIORITY))
         self.refresh()
         self.app.backScreen()
+
+    def editTopic(self):
+        dialog = view_change.EditTopicDialog(self.app, '')
+        urwid.connect_signal(dialog, 'save',
+            lambda button: self.closeEditTopic(dialog, True))
+        urwid.connect_signal(dialog, 'cancel',
+            lambda button: self.closeEditTopic(dialog, False))
+        self.app.popup(dialog)
+
+    def closeEditTopic(self, dialog, save):
+        if save:
+            rows = [row for row in self.change_rows.values() if row.mark]
+            if not rows:
+                pos = self.listbox.focus_position
+                rows = [self.listbox.body[pos]]
+            change_keys = [row.change_key for row in rows]
+            with self.app.db.getSession() as session:
+                for change_key in change_keys:
+                    change = session.getChange(change_key)
+                    change.topic = dialog.entry.edit_text
+                    change.pending_topic = True
+                    self.app.sync.submitTask(
+                        sync.SetTopicTask(change_key, sync.HIGH_PRIORITY))
+        self.app.backScreen()
+        self.refresh()
