@@ -17,6 +17,7 @@ import logging
 
 import urwid
 
+from gertty import gitrepo
 from gertty import keymap
 from gertty import mywid
 from gertty import gitrepo
@@ -105,10 +106,16 @@ class BaseDiffLine(urwid.Button):
     def selectable(self):
         return True
 
+    def search(self, search, attribute):
+        pass
+
 class BaseFileHeader(urwid.Button):
     def selectable(self):
         return True
 
+    def search(self, search, attribute):
+        pass
+
 class BaseFileReminder(urwid.WidgetWrap):
     pass
 
@@ -157,6 +164,8 @@ class BaseDiffView(urwid.WidgetWrap):
              "Add an inline comment"),
             (key(keymap.SELECT_PATCHSETS),
              "Select old/new patchsets to diff"),
+            (key(keymap.INTERACTIVE_SEARCH),
+             "Interactive search"),
             ]
 
     def __init__(self, app, new_revision_key):
@@ -169,6 +178,9 @@ class BaseDiffView(urwid.WidgetWrap):
 
     def _init(self):
         del self._w.contents[:]
+        self.search = None
+        self.results = []
+        self.current_result = None
         with self.app.db.getSession() as session:
             new_revision = session.getRevision(self.new_revision_key)
             old_comments = []
@@ -252,7 +264,7 @@ class BaseDiffView(urwid.WidgetWrap):
                 comment_list.append((comment.key, message))
                 comment_lists[key] = comment_list
                 comment_filenames.add(path)
-        repo = self.app.getRepo(self.project_name)
+        repo = gitrepo.get_repo(self.project_name, self.app.config)
         self._w.contents.append((self.app.header, ('pack', 1)))
         self.file_reminder = self.makeFileReminder()
         self._w.contents.append((self.file_reminder, ('pack', 1)))
@@ -426,10 +438,37 @@ class BaseDiffView(urwid.WidgetWrap):
             context = item.context
         return context
 
+    def search_valid_char(self, ch):
+        return urwid.util.is_wide_char(ch, 0) or (len(ch) == 1 and ord(ch) >= 32)
+
     def keypress(self, size, key):
+        if self.search is not None:
+            if self.search_valid_char(key) or key == 'backspace':
+                if key == 'backspace':
+                    self.search = self.search[:-1]
+                else:
+                    self.search += key
+                self.interactiveSearch(self.search)
+                return None
+            else:
+                commands = self.app.config.keymap.getCommands([key])
+                if keymap.INTERACTIVE_SEARCH in commands:
+                    self.nextSearchResult()
+                    return None
+                else:
+                    self.app.status.update(title=self.title)
+                    if not self.search:
+                        self.interactiveSearch(None)
+                    self.search = None
+                    if key in ['enter', 'esc']:
+                        return None
+
         old_focus = self.listbox.focus
-        r = super(BaseDiffView, self).keypress(size, key)
+        if not self.app.input_buffer:
+            key = super(BaseDiffView, self).keypress(size, key)
         new_focus = self.listbox.focus
+        keys = self.app.input_buffer + [key]
+        commands = self.app.config.keymap.getCommands(keys)
 
         context = self.getContextAtTop(size)
         if context:
@@ -438,14 +477,17 @@ class BaseDiffView(urwid.WidgetWrap):
         else:
             self.file_reminder.set('', '')
 
-        commands = self.app.config.keymap.getCommands(r)
         if (isinstance(old_focus, BaseDiffCommentEdit) and
             (old_focus != new_focus or (keymap.PREV_SCREEN in commands))):
             self.cleanupEdit(old_focus)
         if keymap.SELECT_PATCHSETS in commands:
             self.openPatchsetDialog()
             return None
-        return r
+        if keymap.INTERACTIVE_SEARCH in commands:
+            self.search = ''
+            self.app.status.update(title=("Search: "))
+            return None
+        return key
 
     def mouse_event(self, size, event, button, x, y, focus):
         old_focus = self.listbox.focus
@@ -514,3 +556,23 @@ class BaseDiffView(urwid.WidgetWrap):
         self.app.backScreen()
         self.old_revision_key, self.new_revision_key = dialog.getSelected()
         self._init()
+
+    def interactiveSearch(self, search):
+        if search is not None:
+            self.app.status.update(title=("Search: " + search))
+        self.results = []
+        self.current_result = 0
+        for i, line in enumerate(self.listbox.body):
+            if hasattr(line, 'search'):
+                if line.search(search, 'search-result'):
+                    self.results.append(i)
+
+    def nextSearchResult(self):
+        if not self.results:
+            return
+        dest = self.results[self.current_result]
+        self.listbox.set_focus(dest)
+        self.listbox._invalidate()
+        self.current_result += 1
+        if self.current_result >= len(self.results):
+            self.current_result = 0
