@@ -16,19 +16,21 @@
 import urwid
 
 from gertty import gitrepo
+from gertty import mywid
 from gertty.view.diff import BaseDiffCommentEdit, BaseDiffComment, BaseDiffLine
 from gertty.view.diff import BaseFileHeader, BaseFileReminder, BaseDiffView
 
 LN_COL_WIDTH = 5
 
 class UnifiedDiffCommentEdit(BaseDiffCommentEdit):
-    def __init__(self, context, oldnew, key=None, comment=u''):
+    def __init__(self, app, context, oldnew, key=None, comment=u''):
         super(UnifiedDiffCommentEdit, self).__init__([])
         self.context = context
         self.oldnew = oldnew
         # If we save a comment, the resulting key will be stored here
         self.key = key
-        self.comment = urwid.Edit(edit_text=comment, multiline=True)
+        self.comment = mywid.MyEdit(edit_text=comment, multiline=True,
+                                    ring=app.ring)
         self.contents.append((urwid.Text(u''), ('given', 8, False)))
         self.contents.append((urwid.AttrMap(self.comment, 'draft-comment'),
                               ('weight', 1, False)))
@@ -71,7 +73,8 @@ class UnifiedDiffLine(BaseDiffLine):
             columns = [(LN_COL_WIDTH, urwid.Text(u'')), (LN_COL_WIDTH, new_ln_col)]
         if new_action == ' ':
             columns = [(LN_COL_WIDTH, old_ln_col), (LN_COL_WIDTH, new_ln_col)]
-        line_col = urwid.Text(line)
+        line_col = mywid.SearchableText(line)
+        self.text_widget = line_col
         if action == '':
             line_col = urwid.AttrMap(line_col, 'nonexistent')
         columns += [line_col]
@@ -86,6 +89,9 @@ class UnifiedDiffLine(BaseDiffLine):
                }
         self._w = urwid.AttrMap(col, None, focus_map=map)
 
+    def search(self, search, attribute):
+        return self.text_widget.search(search, attribute)
+
 class UnifiedFileHeader(BaseFileHeader):
     def __init__(self, app, context, oldnew, old, new, callback=None):
         super(UnifiedFileHeader, self).__init__('', on_press=callback)
@@ -135,7 +141,8 @@ class UnifiedDiffView(BaseDiffView):
             old_list = comment_lists.pop(key, [])
             while old_list:
                 (old_comment_key, old_comment) = old_list.pop(0)
-                lines.append(UnifiedDiffCommentEdit(context,
+                lines.append(UnifiedDiffCommentEdit(self.app,
+                                                    context,
                                                     gitrepo.OLD,
                                                     old_comment_key,
                                                     old_comment))
@@ -154,7 +161,8 @@ class UnifiedDiffView(BaseDiffView):
             new_list = comment_lists.pop(key, [])
             while new_list:
                 (new_comment_key, new_comment) = new_list.pop(0)
-                lines.append(UnifiedDiffCommentEdit(context,
+                lines.append(UnifiedDiffCommentEdit(self.app,
+                                                    context,
                                                     gitrepo.NEW,
                                                     new_comment_key,
                                                     new_comment))
@@ -180,7 +188,8 @@ class UnifiedDiffView(BaseDiffView):
         old_list = comment_lists.pop(key, [])
         while old_list:
             (old_comment_key, old_comment) = old_list.pop(0)
-            lines.append(UnifiedDiffCommentEdit(context,
+            lines.append(UnifiedDiffCommentEdit(self.app,
+                                                context,
                                                 gitrepo.OLD,
                                                 old_comment_key,
                                                 old_comment))
@@ -200,14 +209,16 @@ class UnifiedDiffView(BaseDiffView):
         new_list = comment_lists.pop(key, [])
         while new_list:
             (new_comment_key, new_comment) = new_list.pop(0)
-            lines.append(UnifiedDiffCommentEdit(context,
+            lines.append(UnifiedDiffCommentEdit(self.app,
+                                                context,
                                                 gitrepo.NEW,
                                                 new_comment_key,
                                                 new_comment))
         return lines
 
     def makeCommentEdit(self, edit):
-        return UnifiedDiffCommentEdit(edit.context,
+        return UnifiedDiffCommentEdit(self.app,
+                                      edit.context,
                                       edit.oldnew)
 
     def cleanupEdit(self, edit):
